#!/usr/bin/env python
# -*- coding: utf-8 -*-
# author: wuyue92tree@163.com

import os

BASE_DIR = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))

DATEBASE_DIR = os.path.join(BASE_DIR, 'data')

TEMPLATE_DIR = os.path.join(BASE_DIR, 'templates')

CONF_DIR = os.path.join(BASE_DIR, 'crwy')